@@ -28,11 +28,15 @@
 #include FT_TYPE1_TABLES_H
 #include FT_SFNT_NAMES_H
 #include FT_TRUETYPE_IDS_H
+#include FT_MULTIPLE_MASTERS_H
 #include <cairo/cairo-ft.h>
 #include <fontconfig/fontconfig.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <hb.h>
+#include <hb-ot.h>
+#include <hb-ft.h>
 
 #include "font-model.h"
 #include "sushi-font-widget.h"
@@ -73,7 +77,7 @@ _print_version_and_exit (const gchar *option_name,
                          gpointer data,
                          GError **error)
 {
-    g_print("%s %s\n", _("GNOME Font Viewer"), VERSION);
+    g_print("%s %s\n", _("GNOME Fonts"), VERSION);
     exit (EXIT_SUCCESS);
     return TRUE;
 }
@@ -88,6 +92,7 @@ static const GOptionEntry goption_options[] =
 G_DEFINE_TYPE (FontViewApplication, font_view_application, GTK_TYPE_APPLICATION);
 
 static void font_view_application_do_overview (FontViewApplication *self);
+static void ensure_window (FontViewApplication *self);
 
 #define VIEW_ITEM_WIDTH 140
 #define VIEW_ITEM_WRAP_WIDTH 128
@@ -172,18 +177,148 @@ add_row (GtkWidget *grid,
     gtk_label_set_selectable (GTK_LABEL(label), TRUE);
 
     gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+    gtk_label_set_xalign (GTK_LABEL (label), 0.0);
 
-    if (multiline && g_utf8_strlen (value, -1) > 64) {
-        gtk_label_set_width_chars (GTK_LABEL (label), 64);
-        gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    }
+    gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
     gtk_label_set_max_width_chars (GTK_LABEL (label), 64);
+    if (multiline && g_utf8_strlen (value, -1) > 64) {
+       gtk_label_set_width_chars (GTK_LABEL (label), 64);
+    }
+    gtk_label_set_lines (GTK_LABEL (label), multiline ? 10 : 1);
 
     gtk_grid_attach_next_to (GTK_GRID (grid), label, 
                              name_w, GTK_POS_RIGHT,
                              1, 1);
 }
 
+#define FixedToFloat(f) (((float)(f))/65536.0)
+
+static char *
+describe_axis (FT_Var_Axis *ax)
+{
+  /* Translators, this string is used to display information about
+   * a 'font variation axis'. The %s gets replaced with the name
+   * of the axis, for example 'Width'. The three %g get replaced
+   * with the minimum, maximum and default values for the axis.
+   */
+  return g_strdup_printf (_("%s %g — %g, default %g"), ax->name,
+                          FixedToFloat (ax->minimum),
+                          FixedToFloat (ax->maximum),
+                          FixedToFloat (ax->def));
+}
+
+static char *
+get_sfnt_name (FT_Face face,
+               guint id)
+{
+    guint count, i;
+
+    count = FT_Get_Sfnt_Name_Count (face);
+    for (i = 0; i < count; i++) {
+        FT_SfntName sname;
+
+        if (FT_Get_Sfnt_Name (face, i, &sname) != 0)
+            continue;
+
+        if (sname.name_id != id)
+            continue;
+
+        /* only handle the unicode names for US langid */
+        if (!(sname.platform_id == TT_PLATFORM_MICROSOFT &&
+            sname.encoding_id == TT_MS_ID_UNICODE_CS &&
+            sname.language_id == TT_MS_LANGID_ENGLISH_UNITED_STATES))
+            continue;
+
+        return g_convert ((gchar *)sname.string, sname.string_len,
+                          "UTF-8", "UTF-16BE", NULL, NULL, NULL);
+    }
+    return NULL;
+}
+
+/* According to the OpenType spec, valid values for the subfamilyId field
+ * of InstanceRecords are 2, 17 or values in the range (255,32768). See
+ * https://www.microsoft.com/typography/otspec/fvar.htm#instanceRecord
+ */
+static gboolean
+is_valid_subfamily_id (guint id)
+{
+  return id == 2 || id == 17 || (255 < id && id < 32768);
+}
+
+static void
+describe_instance (FT_Face face,
+                   FT_Var_Named_Style *ns,
+                   int pos,
+                   GString *s)
+{
+    char *str = NULL;
+
+    if (is_valid_subfamily_id (ns->strid))
+        str = get_sfnt_name (face, ns->strid);
+
+    if (str == NULL)
+        str = g_strdup_printf (_("Instance %d"), pos);
+
+    if (s->len > 0)
+        g_string_append (s, ", ");
+    g_string_append (s, str);
+
+    g_free (str);
+}
+
+#include "open-type-layout.h"
+
+static char *
+get_features (FT_Face face)
+{
+    hb_font_t *hb_font;
+    int i, j, k;
+    GString *s;
+
+    s = g_string_new ("");
+
+    hb_font = hb_ft_font_create (face, NULL);
+    if (hb_font) {
+        hb_tag_t tables[2] = { HB_OT_TAG_GSUB, HB_OT_TAG_GPOS };
+        hb_face_t *hb_face;
+
+        hb_face = hb_font_get_face (hb_font);
+
+        for (i = 0; i < 2; i++) {
+            hb_tag_t features[80];
+            unsigned int count = G_N_ELEMENTS (features);
+            unsigned int script_index = 0;
+            unsigned int lang_index = HB_OT_LAYOUT_DEFAULT_LANGUAGE_INDEX;
+
+            hb_ot_layout_language_get_feature_tags (hb_face,
+                                                    tables[i],
+                                                    script_index,
+                                                    lang_index,
+                                                    0,
+                                                    &count,
+                                                    features);
+            for (j = 0; j < count; j++) {
+                for (k = 0; k < G_N_ELEMENTS (open_type_layout_features); k++) {
+                    if (open_type_layout_features[k].tag == features[j]) {
+                        if (s->len > 0)
+                            /* Translators, this seperates the list of Layout Features. */
+                            g_string_append (s, C_("OpenType layout", ", "));
+                        g_string_append (s, g_dpgettext2 (NULL, "OpenType layout", open_type_layout_features[k].name));
+                        break;
+                    }
+                }
+            }
+        }
+    }
+
+    if (s->len > 0)
+      return g_string_free (s, FALSE);
+
+    g_string_free (s, TRUE);
+
+    return NULL;
+}
+
 static void
 populate_grid (FontViewApplication *self,
                GtkWidget *grid,
@@ -215,6 +350,7 @@ populate_grid (FontViewApplication *self,
     if (FT_IS_SFNT (face)) {
 	gint i, len;
 	gchar *version = NULL, *copyright = NULL, *description = NULL;
+        gchar *designer = NULL, *manufacturer = NULL, *license = NULL;
 
 	len = FT_Get_Sfnt_Name_Count (face);
 	for (i = 0; i < len; i++) {
@@ -245,6 +381,21 @@ populate_grid (FontViewApplication *self,
 		description = g_convert ((gchar *)sname.string, sname.string_len,
 					 "UTF-8", "UTF-16BE", NULL, NULL, NULL);
 		break;
+	    case TT_NAME_ID_MANUFACTURER:
+		g_free (manufacturer);
+		manufacturer = g_convert ((gchar *)sname.string, sname.string_len,
+			  		 "UTF-8", "UTF-16BE", NULL, NULL, NULL);
+		break;
+	    case TT_NAME_ID_DESIGNER:
+		g_free (designer);
+		designer = g_convert ((gchar *)sname.string, sname.string_len,
+			              "UTF-8", "UTF-16BE", NULL, NULL, NULL);
+		break;
+	    case TT_NAME_ID_LICENSE:
+		g_free (license);
+		license = g_convert ((gchar *)sname.string, sname.string_len,
+		                     "UTF-8", "UTF-16BE", NULL, NULL, NULL);
+		break;
 	    default:
 		break;
 	    }
@@ -264,6 +415,21 @@ populate_grid (FontViewApplication *self,
             add_row (grid, _("Description"), description, TRUE);
 	    g_free (description);
 	}
+	if (manufacturer) {
+            strip_whitespace (&manufacturer);
+            add_row (grid, _("Manufacturer"), manufacturer, TRUE);
+	    g_free (manufacturer);
+	}
+	if (designer) {
+            strip_whitespace (&designer);
+            add_row (grid, _("Designer"), designer, TRUE);
+	    g_free (designer);
+	}
+	if (license) {
+            strip_whitespace (&license);
+            add_row (grid, _("License"), license, TRUE);
+	    g_free (license);
+	}
     } else if (FT_Get_PS_Font_Info (face, &ps_info) == 0) {
         gchar *compressed;
 
@@ -282,6 +448,44 @@ populate_grid (FontViewApplication *self,
     }
 }
 
+static void
+populate_details (FontViewApplication *self,
+                  GtkWidget *grid,
+                  FT_Face face)
+{
+    char *s;
+    FT_MM_Var *ft_mm_var;
+
+    s = g_strdup_printf ("%ld", face->num_glyphs);
+    add_row (grid, _("Glyph Count"), s, FALSE);
+    g_free (s);
+
+    add_row (grid, _("Color Glyphs"), FT_HAS_COLOR (face) ? _("yes") : _("no"), FALSE);
+
+    s = get_features (face);
+    if (s)
+        add_row (grid, _("Layout Features"), s, TRUE);
+    g_free (s);
+
+    if (FT_Get_MM_Var (face, &ft_mm_var) == 0) {
+        int i;
+        for (i = 0; i < ft_mm_var->num_axis; i++) {
+             s = describe_axis (&ft_mm_var->axis[i]);
+             add_row (grid, i == 0 ? _("Variation Axes") : "", s, FALSE);
+             g_free (s);
+        }
+        {
+            GString *str = g_string_new ("");
+            for (i = 0; i < ft_mm_var->num_namedstyles; i++) {
+                 describe_instance (face, &ft_mm_var->namedstyle[i], i, str);
+             }
+             add_row (grid, _("Named Styles"), str->str, TRUE);
+             g_string_free (str, TRUE);
+        }
+        free (ft_mm_var);
+    }
+}
+
 static void
 install_button_refresh_appearance (FontViewApplication *self,
                                    GError *error)
@@ -481,33 +685,47 @@ info_button_clicked_cb (GtkButton *button,
                         gpointer user_data)
 {
     FontViewApplication *self = user_data;
-    GtkWidget *grid, *dialog;
+    GtkWidget *stack, *switcher, *grid, *dialog;
     FT_Face face = sushi_font_widget_get_ft_face (SUSHI_FONT_WIDGET (self->font_widget));
 
     if (face == NULL)
         return;
 
+    dialog = g_object_new (GTK_TYPE_DIALOG,
+                           "transient-for", self->main_window,
+                           "modal", TRUE,
+                           "resizable", FALSE,
+                           "destroy-with-parent", TRUE,
+                           "use-header-bar", TRUE,
+                           NULL);
+    stack = gtk_stack_new ();
+    gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), stack);
+
+    switcher = gtk_stack_switcher_new ();
+    gtk_stack_switcher_set_stack (GTK_STACK_SWITCHER (switcher), GTK_STACK (stack));
+    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (gtk_window_get_titlebar (GTK_WINDOW (dialog))),
+                                     switcher);
+
     grid = gtk_grid_new ();
     gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
-    g_object_set (grid,
-                  "margin-top", 6,
-                  "margin-start", 16,
-                  "margin-end", 16,
-                  "margin-bottom", 6,
-                  NULL);
+    g_object_set (grid, "margin", 20, NULL);
     gtk_grid_set_column_spacing (GTK_GRID (grid), 8);
     gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
 
     populate_grid (self, grid, face);
 
-    dialog = g_object_new (GTK_TYPE_DIALOG,
-                           "title", _("Info"),
-                           "transient-for", self->main_window,
-                           "modal", TRUE,
-                           "destroy-with-parent", TRUE,
-                           "use-header-bar", TRUE,
-                           NULL);
-    gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), grid);
+    gtk_stack_add_titled (GTK_STACK (stack), grid, "info", _("General"));
+
+    grid = gtk_grid_new ();
+    gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+    g_object_set (grid, "margin", 20, NULL);
+    gtk_grid_set_column_spacing (GTK_GRID (grid), 8);
+    gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+
+    populate_details (self, grid, face);
+
+    gtk_stack_add_titled (GTK_STACK (stack), grid, "details", _("Details"));
+
     g_signal_connect (dialog, "response",
                       G_CALLBACK (gtk_widget_destroy), NULL);
     gtk_widget_show_all (dialog);
@@ -531,8 +749,8 @@ font_view_ensure_model (FontViewApplication *self)
     g_signal_connect (self->model, "config-changed",
                       G_CALLBACK (font_model_config_changed_cb), self);
 
-    g_signal_connect (self->main_window, "notify::scale-factor",
-                      G_CALLBACK (font_view_update_scale_factor), self);
+    g_signal_connect_swapped (self->main_window, "notify::scale-factor",
+                              G_CALLBACK (font_view_update_scale_factor), self);
     font_view_update_scale_factor (self);
 }
 
@@ -543,7 +761,6 @@ font_view_application_do_open (FontViewApplication *self,
 {
     GtkWidget *back_image;
     gchar *uri;
-    gboolean rtl;
 
     font_view_ensure_model (self);
 
@@ -572,9 +789,7 @@ font_view_application_do_open (FontViewApplication *self,
 
     if (self->back_button == NULL) {
         self->back_button = gtk_button_new ();
-        rtl = gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL;
-        back_image = gtk_image_new_from_icon_name (rtl ? "go-previous-rtl-symbolic" :
-                                                   "go-previous-symbolic",
+        back_image = gtk_image_new_from_icon_name ("go-previous-symbolic",
                                                    GTK_ICON_SIZE_MENU);
         gtk_button_set_image (GTK_BUTTON (self->back_button), back_image);
         gtk_widget_set_tooltip_text (self->back_button, _("Back"));
@@ -676,6 +891,7 @@ font_view_application_do_overview (FontViewApplication *self)
         GtkCellRenderer *cell;
 
         self->icon_view = icon_view = gtk_icon_view_new_with_model (self->model);
+
         g_object_set (icon_view,
                       "column-spacing", VIEW_COLUMN_SPACING,
                       "margin", VIEW_MARGIN,
@@ -719,9 +935,11 @@ font_view_window_key_press_event_cb (GtkWidget *widget,
                                      GdkEventKey *event,
                                      gpointer user_data)
 {
+    FontViewApplication *self = user_data;
+
     if (event->keyval == GDK_KEY_q &&
         (event->state & GDK_CONTROL_MASK) != 0) {
-        gtk_widget_destroy (widget);
+        g_application_quit (G_APPLICATION (self));
         return TRUE;
     }
 
@@ -737,6 +955,9 @@ query_info_ready_cb (GObject *object,
     GFileInfo *info;
     GError *error = NULL;
 
+    ensure_window (self);
+    g_application_release (G_APPLICATION (self));
+
     info = g_file_query_info_finish (G_FILE (object), res, &error);
     if (error != NULL) {
         font_view_application_do_overview (self);
@@ -756,6 +977,8 @@ font_view_application_open (GApplication *application,
                             const gchar *hint)
 {
     FontViewApplication *self = FONT_VIEW_APPLICATION (application);
+
+    g_application_hold (application);
     g_file_query_info_async (files[0], G_FILE_ATTRIBUTE_STANDARD_NAME,
                              G_FILE_QUERY_INFO_NONE,
                              G_PRIORITY_DEFAULT, NULL,
@@ -786,7 +1009,7 @@ action_about (GSimpleAction *action,
     gtk_show_about_dialog (GTK_WINDOW (self->main_window),
                            "version", VERSION,
                            "authors", authors,
-                           "program-name", _("Font Viewer"),
+                           "program-name", _("Fonts"),
                            "comments", _("View fonts on your system"),
                            "logo-icon-name", "preferences-desktop-font",
                            "translator-credits", _("translator-credits"),
@@ -802,28 +1025,17 @@ static GActionEntry action_entries[] = {
 };
 
 static void
-font_view_application_startup (GApplication *application)
+ensure_window (FontViewApplication *self)
 {
-    FontViewApplication *self = FONT_VIEW_APPLICATION (application);
     GtkWidget *window, *swin;
-    GtkBuilder *builder;
-    GMenuModel *menu;
 
-    G_APPLICATION_CLASS (font_view_application_parent_class)->startup (application);
+    if (self->main_window)
+        return;
 
-    g_action_map_add_action_entries (G_ACTION_MAP (self), action_entries, 
-                                     G_N_ELEMENTS (action_entries), self);
-    builder = gtk_builder_new ();
-    gtk_builder_add_from_resource (builder, "/org/gnome/font-viewer/font-view-app-menu.ui", NULL);
-    menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
-    gtk_application_set_app_menu (GTK_APPLICATION (self), menu);
-
-    g_object_unref (builder);
-    g_object_unref (menu);
-
-    self->main_window = window = gtk_application_window_new (GTK_APPLICATION (application));
+    self->main_window = window = gtk_application_window_new (GTK_APPLICATION (self));
     gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
     gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);
+    gtk_window_set_icon_name (GTK_WINDOW (window), "preferences-desktop-font");
 
     self->header = gtk_header_bar_new ();
     gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header), TRUE);
@@ -856,22 +1068,38 @@ font_view_application_startup (GApplication *application)
     gtk_widget_show_all (window);
 }
 
+static void
+font_view_application_startup (GApplication *application)
+{
+    FontViewApplication *self = FONT_VIEW_APPLICATION (application);
+    GtkBuilder *builder;
+    GMenuModel *menu;
+
+    G_APPLICATION_CLASS (font_view_application_parent_class)->startup (application);
+
+    if (!FcInit ())
+        g_critical ("Can't initialize fontconfig library");
+
+    g_action_map_add_action_entries (G_ACTION_MAP (self), action_entries,
+                                     G_N_ELEMENTS (action_entries), self);
+    builder = gtk_builder_new ();
+    gtk_builder_add_from_resource (builder, "/org/gnome/font-viewer/font-view-app-menu.ui", NULL);
+    menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
+    gtk_application_set_app_menu (GTK_APPLICATION (self), menu);
+
+    g_object_unref (builder);
+    g_object_unref (menu);
+}
+
 static void
 font_view_application_activate (GApplication *application)
 {
     FontViewApplication *self = FONT_VIEW_APPLICATION (application);
 
+    ensure_window (self);
     font_view_application_do_overview (self);
 }
 
-static void
-font_view_application_quit_mainloop (GApplication *application)
-{
-    G_APPLICATION_CLASS (font_view_application_parent_class)->quit_mainloop (application);
-
-    FcFini ();
-}
-
 static void
 font_view_application_dispose (GObject *obj)
 {
@@ -898,7 +1126,6 @@ font_view_application_class_init (FontViewApplicationClass *klass)
     aclass->activate = font_view_application_activate;
     aclass->startup = font_view_application_startup;
     aclass->open = font_view_application_open;
-    aclass->quit_mainloop = font_view_application_quit_mainloop;
 
     oclass->dispose = font_view_application_dispose;
 }
@@ -918,15 +1145,11 @@ main (int argc,
 {
     GApplication *app;
     gint retval;
-    GError *error = NULL;
 
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
 
-    if (!FcInit ())
-        g_critical ("Can't initialize fontconfig library");
-
     app = font_view_application_new ();
     g_application_add_main_option_entries (app, goption_options);
     retval = g_application_run (app, argc, argv);
